// CGI.CPP
// Processes data from an HTML form.
// 11/15/2000 -

// LICENSE
// Copyright (c) 2002-2004, Carson F. Ball
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the following
// conditions are met:
//
// o Redistributions of source code must retain the above copyright
//   notice, this list of conditions, and the following disclaimer.
//
// o Redistributions in binary form must reproduce the above
//   copyright notice, this list of conditions and the following
//   disclaimer in the documentation and/or other materials
//   provided with the distribution.
//
// o Neither the name of the author or of other contributors may be
//   used to endorse or promote products derived from this software
//   without prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
// USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
// AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
// THE POSSIBILITY OF SUCH DAMAGE.

// Define this as being the configuration routines.
#define  CONFIG

#include <ctype.h>   // Needed for isxdigit.
#include <stdio.h>   // Needed for fread.
#include <stdlib.h>  // Needed for getenv.
#include <string.h>  // Needed for strupr.

#include "cgi.h"
#include "misc.h"

// +-------------------------------------------------------------------------+
// |FUNCTION:  CGI                                                           |
// +-------------------------------------------------------------------------+
// |INPUT:     Nothing.                                                      |
// |OUTPUT:    Nothing.                                                      |
// |RETURN:    Nothing.                                                      |
// +-------------------------------------------------------------------------+
// |This function is the constructor for the CGI class.                      |
// +-------------------------------------------------------------------------+
CGI::CGI (void) {
   Status = 0;

   return;
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  ~CGI                                                          |
// +-------------------------------------------------------------------------+
// |INPUT:     Nothing.                                                      |
// |OUTPUT:    Nothing.                                                      |
// |RETURN:    Nothing.                                                      |
// +-------------------------------------------------------------------------+
// |INT32      Count   - A counter used in loops.                            |
// +-------------------------------------------------------------------------+
// |This function is the destructor for the CGI class.                       |
// +-------------------------------------------------------------------------+
CGI::~CGI (void) {
   for (INT32 Count = 0; Count < NumFields; ++Count) {
      delete [] Fields [Count].Field;
      delete [] Fields [Count].Value;
   }

   delete [] Fields;

   return;
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  Get                                                           |
// +-------------------------------------------------------------------------+
// |INPUT:     Nothing.                                                      |
// |OUTPUT:    GetForm - The information returned form the form (or NULL).   |
// |           Num     - The number of fields returned (0 if error).         |
// |           Longest - The length of the longest field name (or error if   |
// |                     the function returns NULL).                         |
// |RETURN:    The number of fields found, or NULL on error.                 |
// +-------------------------------------------------------------------------+
// |char*      Temp    - A temporary string.                                 |
// |INT32      Length  - The length of the data passed from the HTML form.   |
// |INT32      Count   - A counter used in loops.                            |
// +-------------------------------------------------------------------------+
// |This function gets the number of fields POSTed from the form.            |
// +-------------------------------------------------------------------------+
void CGI::Get (void) {
   // Get the method used to send the data (GET or POST).
   INT32 Length;
   char* Temp = getenv ("REQUEST_METHOD");

   // If there is no method name, no form was posted.
   if (!Temp) {
      NumFields = 0;
      Status    = 8;
      Fields    = NULL;
   }

   // Only POST is currently allowed.
   if (!strcmpi (Temp, "POST")) {
      Length = atoi (getenv ("CONTENT_LENGTH"));

      String = new char [Length + 1];
      if (String) {
         String [Length] = 0;

         fread (String, Length, 1, stdin);

         // Convert ASCII codes to characters and remove harmful characters.
         FixStr ();
      }
   }
   else {
      NumFields = 0;
      Status    = 5;
      Fields    = NULL;
   }

   if (!Length || !String) {
      NumFields = 0;
      Status    = 6;
      Fields    = NULL;
   }

   NumField ();
   GetField ();

   Longest      = 0;
   INT32 Count  = 0;

   do {
      if (strlen (Fields [Count].Field) > Longest) {
         Longest = strlen (Fields [Count].Field);
      }
   } while (++Count < NumFields);

   delete [] String;
   return;
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  NumField                                                      |
// +-------------------------------------------------------------------------+
// |INPUT:     String   - The information POSTed from the form.              |
// |OUTPUT:    NumField - The number of valid server addresses.              |
// +-------------------------------------------------------------------------+
// |char*      Pos      - The position within the string.                    |
// +-------------------------------------------------------------------------+
// |This function gets the number of fields POSTed from the form.            |
// +-------------------------------------------------------------------------+
void CGI::NumField () {
   // If there are no fields, exit the function.
   if (!strlen (String)) {
      NumFields = 0;
   }
   else {
      NumFields = 1;      // Initialize the number of fields to 0.
      char* Pos = String; // Set the position to the beginning of the string.

      do {
         // If an ampersand was encountered, increment the number of fields.
         if (*Pos == '&') {
            ++NumFields;
         }
      // Continue while the end of the string has not been reached.
      } while (++Pos < String + strlen (String));
   }

   return;
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  GetField                                                      |
// +-------------------------------------------------------------------------+
// |INPUT:     String   - The information returned from the form.            |
// |           Num      - The number of fields.                              |
// |OUTPUT:    GetField - A pointer to a structure containing the form data. |
// +-------------------------------------------------------------------------+
// |char*      Pos      - The position within the string.                    |
// |INT32      Count    - A counter used in loops.                           |
// |char*      Term     - The position where the substing terminates.        |
// +-------------------------------------------------------------------------+
// |This function gets information POSTed from a form and returns it to the  |
// |calling program in a structure.                                          |
// +-------------------------------------------------------------------------+
void CGI::GetField () {
   char* Pos   = String;
   Fields      = new FIELDS [NumFields];
   INT32 Count = 0;

   // Locate, break apart, and store all fields and their values.
   do {
      // Find the end of a field.
      char* Term = strchr (Pos, '=');

      // Store the field.
      Fields [Count].Field = new char [Term - Pos + 1];
      memcpy (Fields [Count].Field, Pos, Term - Pos);
      Fields [Count].Field [Term - Pos] = 0;
      Pos = Term + 1;

      // Find the end of the field's value.
      Term = strchr (Pos, '&');
      if (!Term) {
         Term = &Pos [strlen (Pos)];
      }

      // Store the field's value.
      Fields [Count].Value = new char [Term - Pos + 1];
      memcpy (Fields [Count].Value, Pos, Term - Pos);
      Fields [Count].Value [Term - Pos] = 0;
      Pos             = Term + 1;
   } while (++Count < NumFields);

   return;
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  FixStr                                                        |
// +-------------------------------------------------------------------------+
// |INPUT:     String   - The information returned from the form.            |
// |OUTPUT:    FixStr   - The "fixed" string.                                |
// +-------------------------------------------------------------------------+
// |INT32      Count    - A counter used in loops.                           |
// |INT8       Char     - The current character to check.                    |
// |bool       Done     - A flag used to indicate whether processing is doen.|
// +-------------------------------------------------------------------------+
// |This function "fixes" the information returned from an HTML form by      |
// |turning plusses into spaces and ASCII codes into ASCII characters.  It   |
// |also turns any unwanted characters into spaces.                          |
// +-------------------------------------------------------------------------+
void CGI::FixStr () {
   // A list of characters that can safely be passed to the server.
   // '=' and '&' are included because they will be parsed out later when the
   // fields and their values are pulled out.  39 is the decimal value of '.
   SCHAR Safe  [] =  {'=', '&', ',',
                      39,  '(', ')', '-', '.', '/', '0', '1', '2', '3', '4',
                      '5', '6', '7', '8', '9', ':', '?', '@', 'A', 'B', 'C',
                      'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N',
                      'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y',
                      'Z', '_', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i',
                      'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't',
                      'u', 'v', 'w', 'x', 'y', 'z'                          };
   INT8  Char;

   INT32 Count = 0;

   // Replace all ASCII escape sequences  with the actual character
   // (e.g. %20 is a space).  All escape sequences are in hexadecimal. Since
   // escape sequences are being converted before safe characters are checked:
   //    1) The results of converted escape sequences are checked.
   //    2) % that do not convert into escape sequences are removed.
   do {
      if (String [Count] == '%') {

         // Are the next two characters hexadecimal digits?
         if (isxdigit (String [Count + 1]) && isxdigit (String [Count + 2])) {

            // Change the four highest bits into a number.
            // Is the digit A-F?

            if (String [Count + 1] > '9') {
               // Subtracting '7' is the same as subtracting 'A' and adding 10.
               Char  = (String [Count + 1] & 0xDF) - '7';
            }
            else {
               // The digit is 0-9.
               Char  = String [Count + 1] - '0';
            }

            // Move the 4 highest bits to their proper places.
            Char <<= 4;

            // Change the four lowest bits into a number.
            // Is the digit A-F?

            if (String [Count + 2] > '9') {
               // Subtracting '7' is the same as subtracting 'A' and adding 10.
               Char += (String [Count + 2] & 0xDF) - '7';
            }
            else {
               // The digit is 0-9.
               Char += String [Count + 2] - '0';
            }

            String [Count] = Char;

            // Remove the two extra places in the string by shifting the
            // characters left.
            memmove (&String [Count + 1],
                     &String [Count + 3],
                     &String [strlen (String)] - &String [Count + 1]);
         }
      }
   } while (++Count < strlen (String));

   // Replace all unsafe characters with a space.  These characters could
   // pose security risks, crash the program, crash the web server, are
   // unreadable, etc.

   Count = 0;

   do {
      if (!strchr (Safe, String [Count])) {
         String [Count] = ' ';
      }
   } while (++Count < strlen (String));


   // &'s break the program because it separates fields on this character.
   // To get around this problem, &'s replaced with spaces; +'s could not be
   // used due to security concerns.  The same problem could occur with =,
   // but this is much less likely.

   // This section solves the above problems.
   Count = strlen (String);

   do {
      // Search for the equal sign.
      if (String [Count] == '=') {
         bool Done = false;

         do {
            // Now search for the matching ampersand.
            if (String [Count] == '&') {
               Done = true;
            }

            Count--;
         } while (!Done && Count);

         ++Count;
      }

      // If an unmatched ampersand.
      else if (String [Count] == '&') {
         String [Count] = '/';
      }

      Count--;
   } while (Count);

   return;
}
