// ERRORS.CPP
// Error display routines.
// 11/15/2000 -

// LICENSE
// Copyright (c) 2002-2004, Carson F. Ball
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the following
// conditions are met:
//
// o Redistributions of source code must retain the above copyright
//   notice, this list of conditions, and the following disclaimer.
//
// o Redistributions in binary form must reproduce the above
//   copyright notice, this list of conditions and the following
//   disclaimer in the documentation and/or other materials
//   provided with the distribution.
//
// o Neither the name of the author or of other contributors may be
//   used to endorse or promote products derived from this software
//   without prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
// USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
// AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
// THE POSSIBILITY OF SUCH DAMAGE.

#include <stdio.h>  // Needed for printf.
#include <stdlib.h> // Needed for exit.
#include "errors.h"
#include "mailio.h"

// The list of error messages.
static const char* EMessage [] = {
   "",
   "The configuration file could not be opened.",
   "No port was listed in the configuration file.",
   "No IP address was listed in the configuration file.",
   "No valid server was listed in the configuration file.",
   "Form data must be sent via the POST method.",
   "No data could be read from the form.",
   "Unauthorized access to this CGI, please inform the webmaster.",
   "This CGI must be called from a form.",
   "No recipient e-mail address was specified in the form.",
   "No subject was specified in the form.",
   "A link was present in the form, but no description was given.",
#ifdef __WIN32__
   "Winsock could not be started.",
#elif __linux__
   "",
#endif
   "Could not open a socket.",
   "Problem connecting to the server.",
   "No domain name was listed in the configuration file."
};

// +-------------------------------------------------------------------------+
// |FUNCTION:  DispError                                                     |
// +-------------------------------------------------------------------------+
// |INPUT:     ECode    - The number of the error to display.                |
// |OUTPUT:    Nothing.                                                      |
// +-------------------------------------------------------------------------+
// |This function displays an error message and exits to the operating       |
// |system with an error code.                                               |
// +-------------------------------------------------------------------------+
void DispError (INT8 ECode) {
   printf ("         ERROR %3u:  %s<BR>\n", ECode, EMessage [ECode]);
   PrnEnd ("Error");
   exit   (ECode);
}
