// MISC.CPP
// Miscellaneous functions.
// 09/11/2002 -

// LICENSE
// Copyright (c) 2002-2004, Carson F. Ball
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the following
// conditions are met:
//
// o Redistributions of source code must retain the above copyright
//   notice, this list of conditions, and the following disclaimer.
//
// o Redistributions in binary form must reproduce the above
//   copyright notice, this list of conditions and the following
//   disclaimer in the documentation and/or other materials
//   provided with the distribution.
//
// o Neither the name of the author or of other contributors may be
//   used to endorse or promote products derived from this software
//   without prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
// USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
// AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
// THE POSSIBILITY OF SUCH DAMAGE.

#include <ctype.h>  // Needed for isalpha.
#include <stdio.h>  // Needed to printf.
#include <string.h> // Needed for strupr.
#include "misc.h"

// +-------------------------------------------------------------------------+
// |FUNCTION:  IsNum                                                         |
// +-------------------------------------------------------------------------+
// |INPUT:     String   - The string to check.                               |
// |OUTPUT:    Nothing.                                                      |
// |RETURN:    True if the string is a number, false otherwise.              |
// +-------------------------------------------------------------------------+
// |char       Value    - The value to return.                               |
// +-------------------------------------------------------------------------+
// |This function checks to see if a string contains only decimal digits and |
// |periods.                                                                 |
// +-------------------------------------------------------------------------+
bool IsNum (const char* String) {
   char Value = true;

   while (*String && Value) {
      if (!(*String == ' ')) {
         if (!isdigit (*String) && *String != '.') {
            Value = false;
         }
      }

      ++String;
   }

   return (Value);
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  License                                                       |
// +-------------------------------------------------------------------------+
// |INPUT:     Nothing.                                                      |
// |OUTPUT:    Nothing.                                                      |
// |RETURN:    Nothing.                                                      |
// +-------------------------------------------------------------------------+
// |This function displays Mailer's license information.                     |
// +-------------------------------------------------------------------------+
void License (void) {
   printf ("LICENSE\n");
   printf ("Copyright (c) 2002-2004, Carson F. Ball\n");
   printf ("All rights reserved.\n\n");
   printf ("Redistribution and use in source and binary forms, with or\n");
   printf ("without modification, are permitted provided that the ");
   printf ("following\nconditions are met:\n\n");
   printf ("o Redistributions of source code must retain the above ");
   printf ("copyright\nnotice, this list of conditions, and the ");
   printf ("following disclaimer.\n\n");
   printf ("o Redistributions in binary form must reproduce the above\n");
   printf ("copyright notice, this list of conditions and the following\n");
   printf ("disclaimer in the documentation and/or other materials\n");
   printf ("provided with the distribution.\n\n");
   printf ("o Neither the name of the author or of other contributors may ");
   printf ("be\nused to endorse or promote products derived from this ");
   printf ("software\nwithout prior written permission.\n\n");
   printf ("THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND\n");
   printf ("CONTRIBUTORS \"AS IS\" AND ANY EXPRESS OR IMPLIED WARRANTIES,\n");
   printf ("INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF\n");
   printf ("MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE\n");
   printf ("DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR\n");
   printf ("CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,\n");
   printf ("SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT ");
   printf ("NOT\nLIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; ");
   printf ("LOSS OF\nUSE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ");
   printf ("HOWEVER CAUSED\nAND ON ANY THEORY OF LIABILITY, WHETHER IN ");
   printf ("CONTRACT, STRICT\nLIABILITY, OR TORT (INCLUDING NEGLIGENCE OR ");
   printf ("OTHERWISE) ARISING\nIN ANY WAY OUT OF THE USE OF THIS ");
   printf ("SOFTWARE, EVEN IF ADVISED OF\nTHE POSSIBILITY OF SUCH DAMAGE.");
   printf ("\n\n");

   return;
}

#ifdef __linux__
// +-------------------------------------------------------------------------+
// |FUNCTION:  strupr                                                        |
// +-------------------------------------------------------------------------+
// |INPUT:     String   - The string to change to upper case.                |
// |OUTPUT:    String   - The string that has been converted to upper case.  |
// |RETURN:    The string that has been converted to upper case.             |
// +-------------------------------------------------------------------------+
// |int        Length   - The length of the string to convert.               |
// +-------------------------------------------------------------------------+
// |This function converts a string to upper case.                           |
// +-------------------------------------------------------------------------+
char* strupr (char* String) {
   int Length = strlen (String);

   for (int Count = 0; Count < Length; ++Count) {
      if (isalpha (String [Count])) {
         String [Count] &= 0xDF;
      }
   }

   return (String);
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  strcmpi                                                       |
// +-------------------------------------------------------------------------+
// |INPUT:     String1  - The first string to compare.                       |
// |OUTPUT:    String2  - The second string to compare.                      |
// |RETURN:    0 if the strings are the same.                                |
// +-------------------------------------------------------------------------+
// |char*      Str1     - A pointer to the first string.                     |
// |char*      Str2     - A pointer to the second string.                    |
// +-------------------------------------------------------------------------+
// |This function compares two strings without regard to case. If the        |
// |strings are identical, 0 is returned.  If String1 is less than String2,  |
// |a negative number is returned.  Otherwise, a positive number is          |
// |returned.                                                                |
// +-------------------------------------------------------------------------+
int strcmpi (const char* String1, const char* String2) {
   const char* Str1 = String1;
   const char* Str2 = String2;

   while (*Str1 && ((*Str1 == *Str2) || ((*Str1 & 0xDF) == (*Str2 & 0xDF)))) {
      ++Str1;
      ++Str2;
  }

  return ((*Str1 & 0xDF) - (*Str2 & 0xDF));
}


#endif
