// MTYPES.H
// Header file describing commonly used types.
// 09/18/2002 -

// LICENSE
// Copyright (c) 2002-2004, Carson F. Ball
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the following
// conditions are met:
//
// o Redistributions of source code must retain the above copyright
//   notice, this list of conditions, and the following disclaimer.
//
// o Redistributions in binary form must reproduce the above
//   copyright notice, this list of conditions and the following
//   disclaimer in the documentation and/or other materials
//   provided with the distribution.
//
// o Neither the name of the author or of other contributors may be
//   used to endorse or promote products derived from this software
//   without prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
// USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
// AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
// THE POSSIBILITY OF SUCH DAMAGE.

#ifndef __MTYPES__
#define __MTYPES__

// Define some values to save typing.
#define SCHAR    static   char   // The definition of a static char.
#define INT8     unsigned char   // The definition of an 8-bit integer.
#define INT16    unsigned short  // The definition of a 16-bit integer.
#define INT32    unsigned int    // The definition of a 32-bit integer.

#ifdef __WIN32__
#define INT64    __int64        // The Windows definition of a 64-bit integer.
#elif  __linux__
#define INT64    long long      // The *ix definition of a 64-bit integer.
#endif

// Structures used in the program.

// The structure to hold a field name and value pair.
struct FIELDS {
   char* Field;       // The field name for the information.
   char* Value;       // The value that the field contains.
};

// The structure to hold the configuration information.
struct CONF {
   char** Valid;      // The list of servers that can run this program.
   char*  ToAddr;     // The e-mail address to send the data to.
   char*  Domain;     // The domain name of the site.
   char*  Subject;    // The subject for the e-mail.
   char*  Link;       // The link to send the user to from the results page.
   char*  Desc;       // The description of the linked page.
   char*  Title;      // The title for the receipt page.
   INT32  Port;       // The port to telnet to.
   char*  IP;         // The IP address to telnet to.
   INT8   NumValid;   // The number of valid server addresses.
};

#endif
