// SOCK.CPP
// SMTP access routines.
// 12/05/2000 -

// LICENSE
// Copyright (c) 2002-2004, Carson F. Ball
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the following
// conditions are met:
//
// o Redistributions of source code must retain the above copyright
//   notice, this list of conditions, and the following disclaimer.
//
// o Redistributions in binary form must reproduce the above
//   copyright notice, this list of conditions and the following
//   disclaimer in the documentation and/or other materials
//   provided with the distribution.
//
// o Neither the name of the author or of other contributors may be
//   used to endorse or promote products derived from this software
//   without prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
// USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
// AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
// THE POSSIBILITY OF SUCH DAMAGE.

#define SOCK

#include <stdio.h>  // Needed for printf.
#include <string.h> // Needed for strlen.

#include "sock.h"
#include "cgi.h"
#include "config.h"
#include "misc.h"

// +-------------------------------------------------------------------------+
// |FUNCTION:  CONNECT                                                       |
// +-------------------------------------------------------------------------+
// |INPUT:     Nothing.                                                      |
// |OUTPUT:    Nothing.                                                      |
// |RETURN:    Nothing.                                                      |
// +-------------------------------------------------------------------------+
// |This function is the constructor for the CONNECT class.                  |
// +-------------------------------------------------------------------------+
CONNECT::CONNECT (void) {
   Status = 0;

#ifdef __WIN32__
   WSAData WData;

   if (WSAStartup (MAKEWORD (1, 1), &WData) != 0) {
      // Winsock could not be started.
      Status = 12;
   }
   else
#endif
   if ((Sock = socket (PF_INET, SOCK_STREAM, 0)) < 0) {
      // The socket could not be created.
      Status = 13;
   }

   return;
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  ~CONNECT                                                      |
// +-------------------------------------------------------------------------+
// |INPUT:     Nothing.                                                      |
// |OUTPUT:    Nothing.                                                      |
// |RETURN:    Nothing.                                                      |
// +-------------------------------------------------------------------------+
// |This function is the destructor for the CONNECT class.                   |
// +-------------------------------------------------------------------------+
CONNECT::~CONNECT (void) {
#ifdef __WIN32__
   closesocket (Sock);
   WSACleanup  ();
#elif __linux__
   close (Sock);
#endif

   return;
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  MailInfo                                                      |
// +-------------------------------------------------------------------------+
// |INPUT:     Fields   - The fields from the form.                          |
// |           Config   - The information from the configuration file.       |
// |           Num      - The number of fields.                              |
// |           Longest  - The length of the longest field.                   |
// |OUTPUT:    Nothing.                                                      |
// |RETURN:    Nothing.                                                      |
// +-------------------------------------------------------------------------+
// |INT32      Count    - A counter used in loops.                           |
// |INT32      Count2   - A counter used in loops.                           |
// +-------------------------------------------------------------------------+
// |This function connects to the mail server and sends the form information.|
// +-------------------------------------------------------------------------+
void CONNECT::MailInfo (FIELDS& Fields, CONF Config, INT32 Num, INT32 Longest) {
   memset (&Server, 0, sizeof (hostent));

   if (IsNum (Config.IP)) {
      Server.sin_addr.s_addr = inet_addr (Config.IP);
   }
   else {
      Host = gethostbyname (Config.IP);
      if (!Host) {
         // The host could not be located.
         Status = 3;

         return;
      }
      else {
         memcpy (&Server.sin_addr.s_addr,
                 Host->h_addr,
                 sizeof (Server.sin_addr.s_addr));
      }
   }

   Server.sin_family      = PF_INET;
   Server.sin_port        = htons (Config.Port);

   if ((connect (Sock, (sockaddr*) &Server, sizeof (Server))) == -1) {
      // Could not connect to the server.
      Status = 4;

      return;
   }
   else {
      Recv ();

      if (Status) {
         // The server is not responding.
         return;
      }
   }

   Send ("HELO ");
   Send (Config.Domain);
   Send ("\n");

   if (Status) {
      // Information could not be sent to the server.
      return;
   }

   Recv ();

   if (Status) {
      // The server is not responding.
      return;
   }

   Send ("MAIL FROM:<Mailer@");
   Send (Config.Domain);
   Send (">\n");

   if (Status) {
      return;
   }

   Recv ();

   if (Status) {
      // The server is not responding.
      return;
   }

   Send ("RCPT TO:<");
   Send (Config.ToAddr);
   Send (">\n");

   if (Status) {
      // Information could not be sent to the server.
      return;
   }

   Recv ();

   if (Status) {
      // The server is not responding.
      return;
   }

   Send ("DATA\n");

   if (Status) {
      // Information could not be sent to the server.
      return;
   }

   Recv ();

   if (Status) {
      // The server is not responding.
      return;
   }

   Send ("SUBJECT:");
   Send (Config.Subject);
   Send ("\n");

   Send ("TO:");
   Send (Config.ToAddr);
   Send ("\n");

   Send ("Content-type: text/html\n");
   Send ("<HTML><BODY>\n");
   Send ("<FONT Face = \"Courier New\" Size = 2>");

   INT32 Count = 0;

   do {
      Send ((&Fields) [Count].Field);

      for (INT32 Count2 = strlen ((&Fields) [Count].Field);
           Count2 < Longest;
           Count2++) {
         Send ("&nbsp;");
      }

      Send ("-><B>");
      Send ((&Fields) [Count].Value);
      Send ("</B><BR>\n");
   } while (++Count < Num);

   Send (".\n");
   Recv ();

   Send ("QUIT\n");
   Recv ();

   printf ("</FONT>\n");
   printf ("</BODY></HTML>\n");

   return;
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  Send                                                          |
// +-------------------------------------------------------------------------+
// |INPUT:     String   - The string to send to the server.                  |
// |           Size     - The size of the string (0 by default).             |
// |OUTPUT:    Nothing.                                                      |
// |RETURN:    Nothing.                                                      |
// +-------------------------------------------------------------------------+
// |This function sends a string to the mail server.  If no size is          |
// |specified, the size is calculated.                                       |
// +-------------------------------------------------------------------------+
void CONNECT::Send (char* String, INT32 Size) {
   if (!Size) {
      Size = strlen (String);
   }

   if (send (Sock, String, Size, MSG_DONTROUTE) < 0) {
      Status = 6;
   }

   return;
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  Recv                                                          |
// +-------------------------------------------------------------------------+
// |INPUT:     Nothing.                                                      |
// |OUTPUT:    Nothing.                                                      |
// |RETURN:    Nothing.                                                      |
// +-------------------------------------------------------------------------+
// |This function receives a string from the mail server and stores it in a  |
// |buffer.                                                                  |
// +-------------------------------------------------------------------------+
void CONNECT::Recv (void) {
   // 0 removes the information from the server after reading it.
   if (recv (Sock, Buffer, BUFLEN, 0) < 0) {
      Status = 5;
   }

   return;
}
